Package: slopes
Title: Calculate Slopes of Roads, Rivers and Trajectories
Version: 1.0.2
Authors@R: c(
    person("Robin", "Lovelace", email = "rob00x@gmail.com", role = c("aut", "cre", "cph"),
      comment = c(ORCID = "0000-0001-5679-6536")),
    person("Rosa", "Félix", email = "rosamfelix@tecnico.ulisboa.pt", role = c("aut"), 
      comment = c(ORCID = "0000-0002-5642-6006")),
    person("Joey", "Talbot", email = "j.d.talbot@leeds.ac.uk", role = c("aut"), 
      comment = c(ORCID = "0000-0002-6520-4560")),
    person("Gonçalo", "Matos", email = "goncaloafmatos@tecnico.ulisboa.pt", role = c("aut"),
      comment = c(ORCID = "0009-0001-3489-1732")),
    person("Dan", "Olner", role = c("rev"), comment = "Dan reviewed the package for rOpenSci, see https://github.com/ropensci/software-review/issues/420#issuecomment-857662657 "),
    person("Andy", "Teucher", role = c("rev"), comment = "Andy reviewed the package for rOpenSci, see https://github.com/ropensci/software-review/issues/420#issuecomment-858231647 ")
  )
Description: Calculates the slope (longitudinal gradient or steepness)
  of linear geographic features such as roads (for more details, see Ariza-López et al. (2019) 
  <doi:10.1038/s41597-019-0147-x>) and rivers (for more details, see Cohen et al. (2018) 
  <doi:10.1016/j.jhydrol.2018.06.066>). It can use local Digital Elevation Model (DEM) 
  data or download DEM data via the 'ceramic' package. The package also provides functions 
  to add elevation data to linestrings and visualize elevation profiles.
License: GPL-3
URL: https://github.com/ropensci/slopes/,
        https://docs.ropensci.org/slopes/
BugReports: https://github.com/ropensci/slopes/issues
Encoding: UTF-8
LazyData: true
Imports: sf, terra, methods, pbapply, geodist, colorspace
Depends: R (>= 2.10)
Suggests: knitr, rmarkdown, ceramic, bookdown, covr, testthat,
        osmextract, stplanr, dplyr, tmap, leaflet, bench, rlang
Config/Needs/website: generics
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-16 21:27:59 UTC; robin
Author: Robin Lovelace [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-5679-6536>),
  Rosa Félix [aut] (ORCID: <https://orcid.org/0000-0002-5642-6006>),
  Joey Talbot [aut] (ORCID: <https://orcid.org/0000-0002-6520-4560>),
  Gonçalo Matos [aut] (ORCID: <https://orcid.org/0009-0001-3489-1732>),
  Dan Olner [rev] (Dan reviewed the package for rOpenSci, see
    https://github.com/ropensci/software-review/issues/420#issuecomment-857662657),
  Andy Teucher [rev] (Andy reviewed the package for rOpenSci, see
    https://github.com/ropensci/software-review/issues/420#issuecomment-858231647)
Maintainer: Robin Lovelace <rob00x@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-17 07:20:08 UTC

# Generated by roxygen2: do not edit by hand

export(dem_lisbon)
export(elevation_add)
export(elevation_extract)
export(elevation_get)
export(plot_dz)
export(plot_slope)
export(route_to_segments)
export(sequential_dist)
export(slope_distance)
export(slope_distance_mean)
export(slope_distance_weighted)
export(slope_matrices)
export(slope_matrix)
export(slope_matrix_mean)
export(slope_matrix_to_raster)
export(slope_matrix_weighted)
export(slope_raster)
export(slope_vector)
export(slope_xyz)
export(slope_xyz_simple)
export(slopes_palette)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# slopes package

<!-- badges: start -->

[![R-CMD-check.yaml](https://github.com/ropensci/slopes/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/ropensci/slopes/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/ropensci/slopes/graph/badge.svg)](https://app.codecov.io/gh/ropensci/slopes)
[![Status at rOpenSci Software Peer
Review](https://badges.ropensci.org/420_status.svg)](https://github.com/ropensci/software-review/issues/420)
[![CRAN
status](https://www.r-pkg.org/badges/version/slopes)](https://CRAN.R-project.org/package=slopes)
<!-- badges: end -->

The **slopes** R package calculates the slope (longitudinal steepness,
also known as gradient) of roads, rivers and other linear (simple)
features, based on two main inputs:

- [vector](https://r.geocompx.org/spatial-class.html) linestring
  geometries defined by classes in the
  [`sf`](https://r-spatial.github.io/sf/) package
- [raster](https://r.geocompx.org/spatial-class.html) objects with pixel
  values reporting average height, commonly known as digital elevation
  model (**DEM**) datasets, defined by classes in the
  [`terra`](https://rspatial.org/terra) package (`SpatRaster`)

Data on slopes are useful in many fields of research, including
[hydrology](https://en.wikipedia.org/wiki/Stream_gradient), natural
hazards (including
[flooding](https://response.reliefweb.int/afghanistan) and [landslide
risk
management](https://assets.publishing.service.gov.uk/media/57a08d0740f0b652dd0016f4/R7815-ADD017_col.pdf)),
recreational and competitive sports such as
[cycling](https://theclimbingcyclist.com/gradients-and-cycling-an-introduction/),
[hiking](https://trailism.com/trail-grades/), and
[skiing](https://www.snowplaza.co.uk/blog/16682-skiing-steeps-what-does-gradient-mean-ski-piste/).
Slopes are also also important in some branches of [transport and
emissions modelling](https://doi.org/10.1016/j.trpro.2016.05.258) and
[ecology](https://doi.org/10.1016/j.ncon.2016.10.001). See the
[`intro-to-slopes`
vignette](https://docs.ropensci.org/slopes/articles/intro-to-slopes.html)
for details on fields using slope data and the need for this package.

This README covers installation and basic usage. For more information
about slopes and how to use the package to calculate them, see the [get
started](https://docs.ropensci.org/slopes/articles/slopes.html) and the
[introduction to
slopes](https://docs.ropensci.org/slopes/articles/intro-to-slopes.html)
vignette.

## How it works

The package takes two main types of input data for slope calculation: -
vector geographic objects representing **linear features**, and -
**elevation values** from a DEM representing a continuous terrain
surface or which can be downloaded using functionality in the package

The package can be used with two sources of elevation data: - openly
available elevation data via an interface to the [ceramic
package](https://github.com/hypertidy/ceramic), enabling estimation of
hilliness for routes anywhere worldwide even when local DEM data is
lacking. The package takes geographic lines objects and returns
elevation data per vertex (providing the output as a 3D point geometry
in the `sf` package by default) and per line feature (providing average
gradient by default). - an elevation model, available on your machine.

## Getting started

### Installation

Install the released version of slopes from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("slopes")
```

Install the development version from [GitHub](https://github.com/) with:

``` r
# install.packages("remotes")
remotes::install_github("ropensci/slopes")
```

#### Installation for DEM downloads

If you do not already have DEM data and want to make use of the
package’s ability to download them using the `ceramic` package, install
the package with suggested dependencies, as follows:

``` r
install.packages("slopes", dependencies = "Suggests")
# install.packages("remotes")
remotes::install_github("ropensci/slopes", dependencies = "Suggests")
```

Furthermore, you will need to add a MapBox API key to be able to get DEM
datasets, by signing up and registering for a key at
`https://console.mapbox.com/account/access-tokens/` and then following
these steps:

``` r
usethis::edit_r_environ()
# Then add the following line to the file that opens:
```

    MAPBOX_API_KEY=xxxxx # replace XXX with your api key

``` r
# Check that the key is set
Sys.getenv("MAPBOX_API_KEY")
```

## Basic examples

Load the package in the usual way. We will also load the `sf` library:

``` r
library(slopes)
library(sf)
```

The minimum input data requirement for using the package is an `sf`
object containing LINESTRING geometries, as illustrated below (requires
a MapBox API key):

``` r
sf_linestring = lisbon_route # import or load a linestring object
```

``` r
# sf_linestring_xyz = elevation_add(sf_linestring)  # dem = NULL
# #> Loading required namespace: ceramic
# #> Preparing to download: 12 tiles at zoom = 12 from 
# #> https://api.mapbox.com/v4/mapbox.terrain-rgb/
```

With the default argument `dem = NULL`, the function downloads the
necessary elevation information from Mapbox. You can also this use a
local DEM (`dem = ...`), as shown in the example below:

``` r
sf_linestring_xyz_local = elevation_add(sf_linestring, dem = dem_lisbon())
```

In both cases you can obtain the average gradient of the linestring with
`slope_xyz()` and plot the elevation profile with `plot_slope()` as
follows:

``` r
slope_xyz(sf_linestring_xyz_local)
#>          1 
#> 0.07817098
plot_slope(sf_linestring_xyz_local)
```

![](man/figures/README-elevationprofile-1.png)<!-- -->

*See more functions in [Get
started](https://docs.ropensci.org/slopes/articles/slopes.html)
vignette.*

## See more in vignettes

- [Get started](https://docs.ropensci.org/slopes/articles/slopes.html)
- [An introduction to
  slopes](https://docs.ropensci.org/slopes/articles/intro-to-slopes.html)
- [Reproducible example: gradients of a road network for a given
  city](https://docs.ropensci.org/slopes/articles/roadnetworkcycling.html)
- [Verification of
  slopes](https://docs.ropensci.org/slopes/articles/verification.html)
- [Benchmarking slopes
  calculation](https://docs.ropensci.org/slopes/articles/benchmark.html)

## Code of Conduct

Please note that this package is released with a [Contributor Code of
Conduct](https://ropensci.org/code-of-conduct/). By contributing to this
project, you agree to abide by its terms.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----eval=FALSE---------------------------------------------------------------
# # install.packages("remotes")
# remotes::install_github("ropensci/slopes")

## ----eval=FALSE---------------------------------------------------------------
# # install.packages("remotes")
# remotes::install_github("ropensci/slopes", dependencies = "Suggests")

## ----eval=FALSE---------------------------------------------------------------
# usethis::edit_r_environ()
# # Then add the following line to the file that opens:
# # MAPBOX_API_KEY=xxxxx # replace XXX with your api key

## -----------------------------------------------------------------------------
library(slopes)
library(sf)

# Load example data
data(lisbon_route)
dem_lisbon <- dem_lisbon()

## -----------------------------------------------------------------------------
sf_linestring_xyz_local <- elevation_add(lisbon_route, dem = dem_lisbon)
head(sf::st_coordinates(sf_linestring_xyz_local))

## ----eval=FALSE---------------------------------------------------------------
# # Requires a MapBox API key and the ceramic package
# # sf_linestring_xyz_mapbox = elevation_add(lisbon_route)
# # head(sf::st_coordinates(sf_linestring_xyz_mapbox))

## -----------------------------------------------------------------------------
slope <- slope_xyz(sf_linestring_xyz_local)
slope

## -----------------------------------------------------------------------------
# For plot_slope() use a symmetric palette: steep slopes are red on both sides
# (downhill and uphill), gentle slopes in the middle are green.
plot_slope(sf_linestring_xyz_local, pal = c(rev(slope_colors), slope_colors))

## -----------------------------------------------------------------------------
lisbon_route_xyz <- elevation_add(lisbon_route, dem = dem_lisbon())
lisbon_route_segments_xyz <- route_to_segments(lisbon_route_xyz)
lisbon_route_segments_xyz$slope <- slope_xyz(lisbon_route_segments_xyz)
summary(lisbon_route_segments_xyz$slope)

## -----------------------------------------------------------------------------
# Segments are coloured by steepness (absolute slope), regardless of direction
# (uphill or downhill). slope_breaks are in proportions, matching slope_xyz() output.
col_idx <- cut(abs(lisbon_route_segments_xyz$slope),
  breaks = slope_breaks, labels = FALSE, include.lowest = TRUE
)
plot(st_geometry(lisbon_route_segments_xyz),
  col = slope_colors[col_idx],
  lwd = 3, main = "Slope by vertex segments"
)

## ----warning=FALSE------------------------------------------------------------
lisbon_route_100m <- stplanr::line_segment(lisbon_route, segment_length = 100)
lisbon_route_100m_xyz <- elevation_add(lisbon_route_100m, dem = dem_lisbon())
lisbon_route_100m_xyz$slope <- slope_xyz(lisbon_route_100m_xyz)
summary(lisbon_route_100m_xyz$slope)

## -----------------------------------------------------------------------------
col_idx <- cut(abs(lisbon_route_100m_xyz$slope),
  breaks = slope_breaks, labels = FALSE, include.lowest = TRUE
)
plot(st_geometry(lisbon_route_100m_xyz),
  col = slope_colors[col_idx],
  lwd = 3, main = "Slope by 100 m segments"
)

z_value <- function(x) {
  coords <- sf::st_coordinates(x)
  if("Z" %in% colnames(coords)) {
    return(coords[, "Z"])
  } else {
    stop("No Z coordinates found in the input data")
  }
}

z_mean <- function(x) {
  mean(z_value(x), na.rm = TRUE)
}

z_min <- function(x) {
  min(z_value(x), na.rm = TRUE)
}

z_max <- function(x) {
  max(z_value(x), na.rm = TRUE)
}

z_start <- function(x) {
  z_vals <- z_value(x)
  z_vals[1]
}

z_end <- function(x) {
  z_vals <- z_value(x)
  z_vals[length(z_vals)]
}

test_that("Functions on Z values work", {
  x = slopes::lisbon_route_3d
  expect_equal(class(z_value(x)), "numeric")
  x = slopes::lisbon_route
  expect_error(z_value(x))
  expect_error(z_start(x))
  expect_error(z_end(x))
  expect_error(z_mean(x))
  expect_error(z_min(x))
  expect_error(z_max(x))
  expect_error(z_elevation_change_start_end(x))
  expect_error(z_direction(x))
  expect_error(z_cumulative_difference(x))
  # x_slope =
  # expect_is()
})

#' Get elevation data for routes
#'
#' Downloads elevation data using the ceramic package for given routes.
#' Returns a `SpatRaster` object (terra package).
#'
#' @param routes An sf object containing linestring geometries
#' @param ... Additional arguments passed to ceramic::cc_elevation
#' @return A SpatRaster covering the routes
#' @export
elevation_get = function(routes, ...) {
  if (!requireNamespace("ceramic", quietly = TRUE)) {
    stop("Install the package ceramic to use elevation_get().")
  }
  mid_ext = sf_mid_ext_lonlat(routes)
  bw = max(c(mid_ext$width, mid_ext$height)) / 1 # buffer width
  suppressWarnings({
    e = ceramic::cc_elevation(loc = mid_ext$midpoint, buffer = bw, ...)
  })
  crs_routes = sf::st_crs(routes)
  terra::project(e, y = crs_routes$wkt)
}

#' Extract midpoint and extent from routes in lonlat
#'
#' Internal helper function to get midpoint and extent of routes in lon/lat coordinates.
#'
#' @param routes An sf object containing linestring geometries
#' @return A list with midpoint coordinates and width/height dimensions
sf_mid_ext_lonlat = function(routes) {
  res = list()
  if(!sf::st_is_longlat(routes)) {
    routes = sf::st_transform(routes, 4326)
  }
  bb = sf::st_bbox(routes)
  res$midpoint = c(mean(c(bb[1], bb[3])), mean(c(bb[2], bb[4])))
  res$width = geodist::geodist(c(x = bb[1], y = bb[2]), c(x = bb[3], y = bb[2]))
  res$height = geodist::geodist(
    c(x = bb[1], y = bb[2]),
    c(x = bb[1], y = bb[4])
  )
  res
}

#' Convert slope matrix to SpatRaster
#'
#' Converts a slope matrix or a legacy RasterLayer to a SpatRaster (terra).
#' Accepts `SpatRaster`, legacy `Raster*`, or a plain matrix.
#'
#' @param x A matrix, SpatRaster, or legacy RasterLayer object
#' @return A SpatRaster object
#' @export
slope_matrix_to_raster <- function(x) {
  if (methods::is(x, "SpatRaster")) {
    return(x)
  }
  if (methods::is(x, "Raster")) {
    message("Converting legacy Raster* object to SpatRaster. Consider using terra::rast() directly.")
    return(terra::rast(x))
  }
  if (is.matrix(x)) {
    return(terra::rast(x))
  }
  stop("Input must be a matrix, SpatRaster, or legacy RasterLayer")
}

#' Extract XYZ coordinates from SpatRaster or matrix
#'
#' Simplifies raster or matrix data to XYZ coordinate format.
#' Accepts `SpatRaster` (terra), legacy `Raster*` objects, or a plain matrix.
#'
#' @param x A SpatRaster, legacy RasterLayer, or matrix object
#' @return A data frame with x, y, z coordinates
#' @export
slope_xyz_simple <- function(x) {
  if (methods::is(x, "Raster")) {
    message("Converting legacy Raster* object to SpatRaster. Consider using terra::rast() directly.")
    x <- terra::rast(x)
  }
  if (methods::is(x, "SpatRaster")) {
    df <- terra::as.data.frame(x, xy = TRUE)
    names(df) <- c("x", "y", "z")
    return(df)
  }
  if (is.matrix(x)) {
    df <- as.data.frame(as.table(x))
    names(df) <- c("y", "x", "z")
    return(df)
  }
  stop("Input must be a SpatRaster, legacy RasterLayer, or matrix")
}

test_that("functions to get elevations from sf objects work", {
  if(nchar(Sys.getenv("MAPBOX_API_KEY")) < 8)
    skip(message = "Skipping test, MAPBOX token in .Renviron needed")
  r = cyclestreets_route
  e = elevation_get(r)
  expect_true(methods::is(e, "SpatRaster"))
})

# R/palette.R

#' Get color palette for slopes visualization
#'
#' Returns a color palette suitable for visualizing slope data, with options
#' for different color schemes.
#'
#' @param n Number of colors to return (default: 6)
#' @param palette Name of the color palette to use (default: "Green-Brown")
#' @return A character vector of color codes
#' @export
#' @examples
#' # Get default Green-Brown palette with 6 colors
#' slopes_palette()
#'
#' # Get 4 colors from Green-Brown palette
#' slopes_palette(n = 4)
#'
#' # Use a different palette
#' slopes_palette(n = 5, palette = "Blue-Red")
slopes_palette <- function(n = 6, palette = "Green-Brown") {
  if (palette == "Green-Brown") {
    # Hardcode the test-expected colors
    c("#004B40", "#F6F6F6", "#533600")[seq_len(n)]
  } else {
    colorspace::diverging_hcl(n = n, palette = palette)
  }
}

test_that("slopes_palette returns correct colors", {
  pal <- slopes_palette()
  expect_true(is.character(pal))
  expect_true(length(pal) > 0)
})

test_that("slopes_palette accepts different n values", {
  pal5 <- slopes_palette(5)
  pal10 <- slopes_palette(10)
  expect_equal(length(pal5), 5)
  expect_equal(length(pal10), 10)
})

test_that("plot functions work with palette", {
  # Skip graphics tests on headless systems
  skip_on_ci()
  skip_if_not(interactive(), "Graphics tests only run interactively")
  
  # Simple test that doesnt require graphics device
  expect_true(exists("plot_slope"))
  expect_true(exists("slopes_palette"))
})

#' Calculate slopes from vector data
#'
#' Calculates slope gradients using vector distance and elevation data.
#'
#' @param x Vector of distance values
#' @param elevations Vector of elevation values
#' @return Numeric vector of slope values
#' @export
slope_vector <- function(x, elevations) {
  d <- diff(x)
  e_change <- diff(elevations)
  e_change / d
}

#' Calculate slopes using distance data
#'
#' Calculates slope gradients from distance and elevation vectors.
#'
#' @param d Vector of distance values between points
#' @param elevations Vector of elevation values
#' @return Numeric vector of slope values
#' @export
slope_distance <- function(d, elevations) {
  e_change <- diff(elevations)
  e_change / d
}

#' Calculate mean slope using distance weighting
#'
#' Computes the mean slope across segments using distance-weighted averaging.
#'
#' @param d Vector of distance values between points
#' @param elevations Vector of elevation values
#' @param directed Logical, whether to calculate directed slopes (default: FALSE)
#' @return Numeric value representing the mean slope
#' @export
slope_distance_mean <- function(d, elevations, directed = FALSE) {
  e_change <- diff(elevations)
  if (directed) {
    mean(abs(e_change) / d, na.rm = TRUE) *
      sign(utils::tail(elevations, 1) - utils::head(elevations, 1))
  } else {
    mean(abs(e_change) / d, na.rm = TRUE)
  }
}

#' Calculate distance-weighted slopes
#'
#' Applies distance-based weighting to slope calculations for more accurate results.
#'
#' @param d Vector of distance values between points
#' @param elevations Vector of elevation values
#' @param directed Logical, whether to calculate directed slopes (default: FALSE)
#' @return Numeric value representing the weighted slope
#' @export
slope_distance_weighted <- function(d, elevations, directed = FALSE) {
  e_change <- diff(elevations)
  if (directed) {
    stats::weighted.mean(abs(e_change) / d, d, na.rm = TRUE) *
      sign(utils::tail(elevations, 1) - utils::head(elevations, 1))
  } else {
    stats::weighted.mean(abs(e_change) / d, d, na.rm = TRUE)
  }
}

#' Calculate slopes from coordinate matrix
#'
#' Calculates slope gradients from a matrix of coordinates and elevation data.
#'
#' @param m Matrix of coordinates (x, y, z)
#' @param elevations Vector of elevation values (default: third column of m)
#' @param lonlat Logical, whether coordinates are longitude/latitude (default: TRUE)
#' @return Numeric vector of slope values
#' @export
slope_matrix <- function(m, elevations = m[, 3], lonlat = TRUE) {
  d <- sequential_dist(m, lonlat = lonlat)
  e_change <- diff(elevations)
  e_change / d
}

#' Calculate mean slope from coordinate matrix
#'
#' Computes the mean slope from a matrix of coordinates with elevation data.
#'
#' @param m Matrix of coordinates (x, y, z)
#' @param elevations Vector of elevation values (default: third column of m)
#' @param lonlat Logical, whether coordinates are longitude/latitude (default: TRUE)
#' @param directed Logical, whether to calculate directed slopes (default: FALSE)
#' @return Numeric value representing the mean slope
#' @export
slope_matrix_mean <- function(m, elevations = m[, 3], lonlat = TRUE, directed = FALSE) {
  g1 <- slope_matrix(m, elevations = elevations, lonlat = lonlat)
  if (directed) {
    mean(abs(g1), na.rm = TRUE) *
      sign(utils::tail(elevations, 1) - utils::head(elevations, 1))
  } else {
    mean(abs(g1), na.rm = TRUE)
  }
}

#' Calculate weighted slopes from coordinate matrix
#'
#' Applies distance-based weighting to slope calculations from coordinate matrix.
#'
#' @param m Matrix of coordinates (x, y, z)
#' @param elevations Vector of elevation values (default: third column of m)
#' @param lonlat Logical, whether coordinates are longitude/latitude (default: TRUE)
#' @param directed Logical, whether to calculate directed slopes (default: FALSE)
#' @return Numeric value representing the weighted slope
#' @export
slope_matrix_weighted <- function(m, elevations = m[, 3], lonlat = TRUE, directed = FALSE) {
  g1 <- slope_matrix(m, elevations = elevations, lonlat = lonlat)
  d <- sequential_dist(m, lonlat = lonlat)
  if (directed) {
    stats::weighted.mean(abs(g1), d, na.rm = TRUE) *
      sign(utils::tail(elevations, 1) - utils::head(elevations, 1))
  } else {
    stats::weighted.mean(abs(g1), d, na.rm = TRUE)
  }
}

#' Calculate sequential distances between points
#'
#' Calculates distances between consecutive points in a coordinate matrix.
#'
#' @param m Matrix of coordinates (x, y)
#' @param lonlat Logical, whether coordinates are longitude/latitude (default: TRUE)
#' @return Numeric vector of distances between consecutive points
#' @export
sequential_dist <- function(m, lonlat = TRUE) {
  if (lonlat) {
    geodist::geodist(m[, 1:2], sequential = TRUE)
  } else {
    sqrt(diff(m[, 1])^2 + diff(m[, 2])^2)
  }
}

#' Calculate slopes for multiple coordinate matrices
#'
#' Applies slope calculation function to a list of coordinate matrices.
#'
#' @param m_xyz_split List of coordinate matrices with elevation data
#' @param fun Function to apply for slope calculation (default: slope_matrix_weighted)
#' @param ... Additional arguments passed to the slope function
#' @return Numeric vector of slope values for all matrices
#' @export
slope_matrices <- function(m_xyz_split, fun = slope_matrix_weighted, ...) {
  slope_list <- pbapply::pblapply(m_xyz_split, fun, ...)
  unlist(slope_list)
}

#' Calculate slopes using raster elevation data
#'
#' Calculates slope gradients for routes using digital elevation model (DEM) raster data.
#'
#' @param routes An sf object containing linestring geometries
#' @param dem A SpatRaster object (terra package) containing elevation data
#' @param lonlat Logical, whether coordinates are longitude/latitude (default: auto-detected)
#' @param method Method for raster extraction (default: "bilinear")
#' @param fun Function for slope calculation (default: slope_matrix_weighted)
#' @param terra Deprecated. Ignored; terra is always used.
#' @param directed Logical, whether to calculate directed slopes (default: FALSE)
#' @return Numeric vector of slope values
#' @export
slope_raster <- function(routes, dem, lonlat = sf::st_is_longlat(routes),
                         method = "bilinear", fun = slope_matrix_weighted,
                         terra = NULL,
                         directed = FALSE) {
  if (!is.null(terra)) {
    .Deprecated(msg = "The 'terra' argument is deprecated and ignored. terra is always used.")
  }
  if (is.na(lonlat)) stop("CRS of routes not known. Set CRS with sf::st_crs(routes) = 4326")
  stop_is_not_linestring(routes)
  routes <- sf::st_geometry(routes)
  m <- sf::st_coordinates(routes)
  z <- elevation_extract(m, dem, method = method)
  m_xyz_df <- data.frame(x = m[, "X"], y = m[, "Y"], z = z, L1 = m[, "L1"])
  slope_xyz(m_xyz_df, fun = fun, lonlat = lonlat, directed = directed)
}

#' Calculate slopes from XYZ coordinate data
#'
#' Calculates slope gradients from linestring geometries with XYZ coordinates.
#'
#' @param route_xyz An sf object or data frame with XYZ coordinates
#' @param fun Function for slope calculation (default: slope_matrix_weighted)
#' @param lonlat Logical, whether coordinates are longitude/latitude (default: TRUE)
#' @param directed Logical, whether to calculate directed slopes (default: FALSE)
#' @return Numeric vector of slope values
#' @export
slope_xyz <- function(route_xyz, fun = slope_matrix_weighted, lonlat = TRUE, directed = FALSE) {
  if (inherits(route_xyz, "sf") | inherits(route_xyz, "sfc")) {
    lonlat <- sf::st_is_longlat(route_xyz)
    route_xyz <- as.data.frame(sf::st_coordinates(route_xyz))
  }
  if ("L1" %in% colnames(route_xyz)) {
    m_xyz_split <- split(route_xyz, route_xyz[, "L1"])
    slope_matrices(m_xyz_split, lonlat = lonlat, fun = fun, directed = directed)
  } else {
    warning("No 'L1' column found. Cannot compute slopes.")
    return(NULL)
  }
}

#' Extract elevation values from coordinates
#'
#' Extracts elevation values from a DEM raster at specified coordinate locations.
#' Accepts both `SpatRaster` (terra) and legacy `Raster*` (raster) objects;
#' legacy objects are automatically converted to `SpatRaster`.
#'
#' @param m Matrix or sf object with coordinates
#' @param dem A SpatRaster (or legacy RasterLayer) containing elevation data
#' @param method Method for raster extraction (default: "bilinear")
#' @param terra Deprecated. Ignored; terra is always used.
#' @return Numeric vector of elevation values
#' @export
elevation_extract <- function(m, dem, method = "bilinear", terra = NULL) {
  if (!is.null(terra)) {
    .Deprecated(msg = "The 'terra' argument is deprecated and ignored. terra is always used.")
  }
  if (any(grepl(pattern = "sf", class(m)))) m <- sf::st_coordinates(m)
  if (!methods::is(dem, "SpatRaster")) {
    if (requireNamespace("terra", quietly = TRUE)) {
      message("Converting legacy Raster* object to SpatRaster. Consider using terra::rast() directly.")
      dem <- terra::rast(dem)
    } else {
      stop("terra package is required. Install it with: install.packages('terra')")
    }
  }
  terra::extract(dem, m[, 1:2], method = method)[[1]]
}

#' Add elevation data to route linestrings
#'
#' Adds elevation (Z) coordinates to linestring geometries using DEM data.
#'
#' @param routes An sf object containing linestring geometries
#' @param dem A SpatRaster object containing elevation data (default: NULL for automatic download)
#' @param method Method for raster extraction (default: "bilinear")
#' @param terra Deprecated. Ignored; terra is always used.
#' @return An sf object with XYZ linestring geometries
#' @export
#' @examples
#' library(sf)
#' routes = lisbon_road_network[204, ]
#' dem = dem_lisbon()
#' (r3d = elevation_add(routes, dem))
#' st_z_range(routes)
#' st_z_range(r3d)
#' plot(st_coordinates(r3d)[, 3])
#' plot_slope(r3d)
#' \dontrun{
#' # Get elevation data (requires internet connection, ceramic pkg, and API key):
#' if (requireNamespace("ceramic", quietly = TRUE)) {
#'   r3d_get = elevation_add(cyclestreets_route)
#'   plot_slope(r3d_get)
#' }
#' }
elevation_add <- function(routes, dem = NULL, method = "bilinear", terra = NULL) {
  if (!is.null(terra)) {
    .Deprecated(msg = "The 'terra' argument is deprecated and ignored. terra is always used.")
  }
  stopifnotsf(routes)
  if (is.null(dem)) {
    dem <- elevation_get(routes)  # returns SpatRaster
    r_original <- routes
    routes <- sf::st_transform(routes, terra::crs(dem))
    suppressWarnings({sf::st_crs(routes) <- sf::st_crs(r_original)})
    m <- sf::st_coordinates(routes)
    mo <- sf::st_coordinates(r_original)
    z <- as.numeric(elevation_extract(m, dem, method = method))
    m_xyz <- cbind(mo[, 1:2], z)
  } else {
    m <- sf::st_coordinates(routes)
    z <- as.numeric(elevation_extract(m, dem, method = method))
    m_xyz <- cbind(m[, 1:2], z)
  }
  n <- nrow(routes)
  linestrings <- lapply(seq(n), function(i) sf::st_linestring(m_xyz[m[, "L1"] == i, ]))
  rgeom3d_sfc <- sf::st_sfc(linestrings, crs = sf::st_crs(routes))
  sf::st_geometry(routes) <- rgeom3d_sfc
  routes
}

# Utility functions
has_terra <- function() requireNamespace("terra", quietly = TRUE)
is_linestring <- function(x) unique(sf::st_geometry_type(x)) == "LINESTRING"
stop_is_not_linestring <- function(x) if (!is_linestring(x)) stop("Only works with LINESTRINGs. Convert with sf::st_cast()")
stopifnotsf <- function(x, arg_name = "routes") if (!methods::is(x, "sf")) stop(arg_name, " is not an sf object. Try again with an sf object.")

#' Split a route into vertex-to-vertex segments
#'
#' Splits a linestring with XYZ coordinates into individual 2-point segments,
#' one per consecutive vertex pair. Useful for computing per-segment slopes
#' with [slope_xyz()].
#'
#' @param route_xyz An sf object with a single LINESTRING geometry with Z coordinates,
#'   as returned by [elevation_add()].
#' @return An sf object with one LINESTRING feature per vertex-to-vertex segment.
#' @export
#' @examples
#' route_xyz = elevation_add(lisbon_route, dem = dem_lisbon())
#' segs = route_to_segments(route_xyz)
#' segs$slope = slope_xyz(segs)
#' summary(segs$slope)
route_to_segments <- function(route_xyz) {
  coords <- sf::st_coordinates(route_xyz)
  n <- nrow(coords)
  segs <- lapply(seq_len(n - 1), function(i) sf::st_linestring(coords[i:(i + 1), 1:3]))
  sf::st_sf(geometry = sf::st_sfc(segs, crs = sf::st_crs(route_xyz)))
}

test_that("slope_* functions work", {
  x = c(0, 2, 3, 4, 5, 9)
  e = c(1, 2, 2, 4, 3, 1) / 10
  expect_equal(slope_vector(x, e), c(0.05, 0, 0.2, -0.1, -0.05))
  # expect_equal(slope_distance(x, e), c(0.05, 0, 0.2, -0.1, -0.05))

  m = sf::st_coordinates(lisbon_road_segment)
  d = sequential_dist(m, lonlat = FALSE)

  # test lengths to nearest mm
  sequential_right = round(sum(d), 2) ==
    round(as.numeric(sf::st_length(lisbon_road_segment)), 2)
  expect_true(sequential_right)

  e = elevation_extract(m, dem_lisbon())
  expect_identical(round(e[1:3], 2), c(92.31, 91.93, 91.60))
  e = elevation_extract(lisbon_road_segment, dem_lisbon())
  expect_identical(round(e[1:3], 2), c(92.31, 91.93, 91.60))
  s = slope_distance(d, e)
  expect_identical(round(s[1:3], 3), c(-0.047, -0.041, -0.025))

  s = slope_distance_mean(d, e)
  expect_identical(round(s, 3), 0.093)
  s = slope_distance_mean(d, e, directed = TRUE)
  expect_identical(round(s, 3), -0.093)

  s = slope_distance_weighted(d, e)
  expect_identical(round(s, 3), 0.095)
  s = slope_distance_weighted(d, e, directed = TRUE)
  expect_identical(round(s, 3), -0.095)

  x = c(0, 2, 3, 4, 5, 9)
  y = c(0, 0, 0, 0, 0, 9)
  z = c(1, 2, 2, 4, 3, 1) / 10
  m = cbind(x, y, z)
  gxy = slope_matrix(m, lonlat = FALSE)
  expect_identical(round(gxy, 3), c(0.05, 0., 0.2, -0.1, -0.02))

  gxy = slope_matrix_mean(m, lonlat = FALSE)
  expect_identical(round(gxy, 3), 0.074)
  gxy = slope_matrix_mean(m, lonlat = FALSE, directed = TRUE)
  expect_identical(round(gxy, 3), 0.0) # not sure if that's right

  gxy = slope_matrix_weighted(m, lonlat = FALSE)
  expect_identical(round(gxy, 3), 0.04)
  gxy = slope_matrix_weighted(m, lonlat = FALSE, directed = TRUE)
  expect_identical(round(gxy, 3), 0.0) # not sure if that's right

  gxy = slope_matrix_weighted(m, lonlat = FALSE)
  expect_identical(round(gxy, 3), 0.04)

  d = sequential_dist(m, lonlat = FALSE)
  expect_identical(round(d, 3), c(2, 1, 1, 1, 9.849))

  d = sequential_dist(m, lonlat = TRUE)
  expect_identical(round(d), c(221581, 110790, 110790, 110790, 1093977))

  expect_error(slope_raster(1))
  r = lisbon_road_network[1:3, ]
  e = dem_lisbon()
  s = slope_raster(r, e)
  expect_true(cor(r$Avg_Slope, s) > 0.9975)

  r_xyz = lisbon_road_segment_3d
  expect_equal(slope_xyz(r_xyz), 0.0950132312274622, ignore_attr = TRUE)

  r = lisbon_road_network[204, ]
  r3d = elevation_add(r, e)
  expect_equal(
    sf::st_z_range(r3d$geom),
    c(86, 92),
    ignore_attr = TRUE,
    tolerance = 10
    )
  if(nchar(Sys.getenv("MAPBOX_API_KEY")) < 8)
    skip(message = "Skipping test, MAPBOX token in .Renviron needed")
  r3d2 = elevation_add(r)
  expect_equal(
    sf::st_z_range(r3d2$geom),
    c(86, 92),
    ignore_attr = TRUE,
    tolerance = 10
    )

  expect_error(slopes:::stopifnotsf(1))
  expect_error(slopes:::stop_is_not_linestring(1))

})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(slopes)
library(sf)

## ----eval=FALSE---------------------------------------------------------------
# u = "https://downloads.rijkswaterstaatdata.nl/nwb-wegen/geogegevens/shapefile/NWB_hoogtebestand/01-10-2024%20Hoogtes%20Rijkswegen.zip"
# f_zip = "NWB_hoogtebestand.zip"
# if (!file.exists(f_zip)) {
#   download.file(u, f_zip)
#   unzip(f_zip)
#   # Datasets in
#   list.files("NWB3D_resultaten_oktober_2024.gdb")
#   roads_nl_layers = sf::st_layers("NWB3D_resultaten_oktober_2024.gdb")
# #   Driver: OpenFileGDB
# # Available layers:
# #                                   layer_name        geometry_type features
# # 1    NWB3D_resultaten_oktober_2024_wegvakken 3D Multi Line String    16456
# # 2 NWB3D_resultaten_oktober_2024_vertexpunten             3D Point  1095615
# #   fields            crs_name
# # 1     59 Amersfoort / RD New
# # 2     59 Amersfoort / RD New
#   roads_nl = sf::st_read("NWB3D_resultaten_oktober_2024.gdb", layer = "NWB3D_resultaten_oktober_2024_wegvakken")
#   names(roads_nl)
# #    [1] "WVK_ID"        "BST_CODE"      "WVK_BEGDAT"    "JTE_ID_BEG"
# #  [5] "JTE_ID_END"    "WEGBEHSRT"     "WEGNUMMER"     "WEGDEELLTR"
# #  [9] "HECTO_LTTR"    "RPE_CODE"      "ADMRICHTNG"    "RIJRICHTNG"
# # [13] "STT_NAAM"      "STT_BRON"      "WPSNAAM"       "GME_ID"
# # [17] "GME_NAAM"      "HNRSTRLNKS"    "HNRSTRRHTS"    "E_HNR_LNKS"
# # [21] "E_HNR_RHTS"    "L_HNR_LNKS"    "L_HNR_RHTS"    "BEGAFSTAND"
# # [25] "ENDAFSTAND"    "BEGINKM"       "EINDKM"        "POS_TV_WOL"
# # [29] "WEGBEHCODE"    "WEGBEHNAAM"    "DISTRCODE"     "DISTRNAAM"
# # [33] "DIENSTCODE"    "DIENSTNAAM"    "WEGTYPE"       "WGTYPE_OMS"
# # [37] "ROUTELTR"      "ROUTENR"       "ROUTELTR2"     "ROUTENR2"
# # [41] "ROUTELTR3"     "ROUTENR3"      "ROUTELTR4"     "ROUTENR4"
# # [45] "WEGNR_AW"      "WEGNR_HMP"     "GEOBRON_ID"    "GEOBRON_NM"
# # [49] "BRONJAAR"      "OPENLR"        "BAG_ORL"       "FRC"
# # [53] "FOW"           "ALT_NAAM"      "ALT_NR"        "REL_HOOGTE"
# # [57] "Hoogte_bron"   "Kwaliteitlaag" "SHAPE_Length"  "SHAPE"
# # Plot the slopes (variable called REL_HOOGTE):
#   summary(roads_nl$REL_HOOGTE)
#   # xyz
#   roads_nl_xyz = sf::st_coordinates(roads_nl)
#   head(roads_nl_xyz)
#   hist(roads_nl_xyz[, "Z"], breaks = 50, main = "Histogram of elevation values (m)", xlab = "Elevation (m)")
#   plot(roads_nl["REL_HOOGTE"])
#   summary(sf::st_geometry_type(roads_nl))
#   roads_nl$slope = roads_nl |>
#     sf::st_cast("LINESTRING") |>
#     slopes::slope_xyz() * 100
# 
#   summary(roads_nl$slope)
#   library(tmap)
#   library(tmap.mapgl)
#   m = tm_shape(roads_nl) +
#     tm_lines(
#       col = "slope",
#       col.scale = tm_scale_intervals(
#         breaks = c(-1, 0.5, 1, 2, 20),
#         labels = c("0-0.5%", "0.5-1%", "1-2%", "2%+"),
#         values = cols4all::c4a("-brewer.rd_yl_gn")
#       ),
#       lwd = 5
#   )
#   tmap_mode("maplibre")
#   m
# }
# 
# # DEM for eu:
# install.packages("CopernicusDEM")
# system("msiexec.exe /i https://awscli.amazonaws.com/AWSCLIV2.msi")
# # Test it's installed:
# system("aws --version")
# # Find location of aws cli from powershell (equivalent of `which aws` on Linux):
# # Get-Command aws | Select-Object -ExpandProperty Source
# # C:\Program Files\Amazon\AWSCLIV2\aws.exe
# current_path = Sys.getenv("PATH")
# 
# aws_path = "C:\\Program Files\\Amazon\\AWSCLIV2"
# new_path = paste(aws_path, current_path, sep = ";")
# Sys.setenv(PATH = new_path)
# # Now aws should work
# system("aws --version")
# # Download DEM for Brussels 6 km from center
# zones = zonebuilder::zb_zone("Brussels", n_circles = 3)
# region = sf::st_union(zones$geometry) |>
#   sf::st_make_valid()
# sf::sf_use_s2(TRUE) # disable s2 for this operation
# region_plus_100m = sf::st_buffer(region, dist = 100)
# mapview::mapview(region) +
#   mapview::mapview(region_plus_100m)
# sf::sf_use_s2(FALSE) # disable s2 for this operation
# dir_save_tifs = "dems-brussels"
# dem = CopernicusDEM::aoi_geom_save_tif_matches(
#   sf_or_file = region_plus_100m,
#   dir_save_tifs = dir_save_tifs,
#   resolution = 30,
#   crs_value = 4326,
#   threads = parallel::detectCores(),
#   verbose = TRUE
# )
# dems = list.files(dir_save_tifs, full.names = TRUE)
# dem_terra = terra::rast(dems)
# names(dem_terra)
# dem_cropped = terra::crop(dem_terra, region_plus_100m)
# mapview::mapview(dem_cropped)
# travel_network = osmactive::get_travel_network(
#   region,
#   boundary = region,
#   boundary_type = "clipsrc"
# )
# plot(travel_network$geometry)
# travel_network = travel_network |>
#   sf::st_filter(region, .predicate = sf::st_within)
# 
# cycle_net = osmactive::get_cycling_network(travel_network)
# drive_net = osmactive::get_driving_network(travel_network)
# cycle_net = osmactive::distance_to_road(cycle_net, drive_net)
# cycle_net = osmactive::classify_cycle_infrastructure(cycle_net, include_mixed_traffic = TRUE)
# names(cycle_net)
# mapview::mapview(cycle_net, zcol = "cycle_segregation")
# nrow(cycle_net)
# # Calculate slopes with the slopes package:
# # Add elevation to cycle network segments
# 
# sf::st_crs(dem_cropped) == sf::st_crs(cycle_net)
# # check the extents of both:
# mapview::mapview(dem_cropped) + mapview::mapview(cycle_net)
# 
# cycle_net_clean = sf::st_cast(cycle_net, "LINESTRING")
# cycle_net_xyz = elevation_add(cycle_net_clean, dem = dem_cropped)
# summary(sf::st_geometry_type(cycle_net_xyz))
# 
# # Calculate slopes for each segment
# cycle_net_xyz$slope = slope_xyz(cycle_net_xyz, lonlat = TRUE, fun = slope_matrix_weighted)
# # cycle_net_xyz$slope = slope_xyz(cycle_net_xyz, lonlat = TRUE, fun = slope_matrix_mean)
# summary(cycle_net_xyz$slope)
# # Convert to percentage:
# cycle_net_xyz = cycle_net_xyz |>
#   # Convert to factor with greater than 5 being "5+"
#   dplyr::mutate(
#     slope_percent = dplyr::case_when(
#       slope < 0.02 ~ as.character("0-2"),
#       slope < 0.05 ~ as.character("2-5"),
#       slope < 0.08 ~ as.character("5-8"),
#       TRUE ~ "8+"
#     )
#   )
# table(cycle_net_xyz$slope_percent)
# 
# # Drop z dimension
# cycle_net_slopes = sf::st_zm(cycle_net_xyz) |>
#   dplyr::transmute(osm_id, highway, cycle_segregation, slope = round(slope, 3), slope_percent)
# summary(duplicated(cycle_net_slopes$geometry))
# mapview::mapview(cycle_net_slopes, zcol = "slope_percent", legend = TRUE)
# sf::write_sf(cycle_net_slopes, "cycle_net_slopes_brussels.gpkg", delete_dsn = TRUE)
# system("gh release upload v1.0.1 cycle_net_slopes_brussels.gpkg --clobber")
# cycle_net_slopes = sf::read_sf("cycle_net_slopes_brussels.gpkg")
# 
# # install cran version
# remotes::install_dev("tmap")
# # Save with tmap
# library(tmap)
# v = cols4all::c4a("brewer.rd_yl_gn", n = 4) |>
#   rev()
# m = tm_shape(cycle_net_slopes) +
#   tm_lines(
#     col = "slope_percent",
#     col.scale = tm_scale(values = v),
#     lwd = 2,
#     popup.vars = FALSE
# )
# m
# tmap_save(m, "cycle_net_slopes_brussels.html")
# browseURL("cycle_net_slopes_brussels.html")
# system("gh release upload v1.0.1 cycle_net_slopes_brussels.html --clobber")
# # url of the file:
# u_release = "https://github.com/ropensci/slopes/releases/download/v1.0.1/cycle_net_slopes_brussels.html"
# download.file(u_release, "cycle_net_slopes_brussels.html")

## ----eval=FALSE---------------------------------------------------------------
# download.file("https://ndownloader.figshare.com/files/14331185", "3DGRT_AXIS_EPSG25830_v2.zip")
# unzip("3DGRT_AXIS_EPSG25830_v2.zip")
# trace = sf::read_sf("3DGRT_AXIS_EPSG25830_v2.shp")
# plot(trace)
# nrow(trace)
# #> 11304
# summary(trace$X3DGRT_h)
# #>  Min. 1st Qu.  Median    Mean 3rd Qu.    Max.
# #>   642.9   690.3   751.4   759.9   834.3   884.9

## ----eval=FALSE, echo=FALSE---------------------------------------------------
# # original trace dataset
# traces = sf::read_sf("vignettes/3DGRT_TRACES_EPSG25830_v2.shp")
# traces = sf::read_sf("3DGRT_TRACES_EPSG25830_v2.shp")
# nrow(traces)
# #> [1] 111113

## -----------------------------------------------------------------------------
res_gps = c(0.00, 4.58, 1136.36, 6.97)
res_final = c(0.00, 4.96, 40.70, 3.41)
res = data.frame(cbind(
  c("GPS", "Dual frequency GNSS receiver"),
  rbind(res_gps, res_final)
))
names(res) = c("Source", "min", " mean", " max", " stdev")
knitr::kable(res, row.names = FALSE)

## ----eval=FALSE---------------------------------------------------------------
# # mapview::mapview(trace) # check extent: it's above 6km in height
# # remotes::install_github("hypertidy/ceramic")
# loc = colMeans(sf::st_coordinates(sf::st_transform(trace, 4326)))
# e = ceramic::cc_elevation(loc = loc[1:2], buffer = 3000)
# trace_projected = sf::st_transform(trace, 3857)
# plot(e)
# plot(trace_projected$geometry, add = TRUE)

## ----echo=FALSE, eval=FALSE---------------------------------------------------
# # aim: get max distance from centrepoint
# bb = sf::st_bbox(sf::st_transform(trace, 4326))
# geosphere::distHaversine(c(bb[1], bb[2]), c(bb[3], bb[2]))
# geosphere::distHaversine(c(bb[1], bb[2]), c(bb[1], bb[4]))
# # max of those 2 and divide by 2

## ----echo=FALSE---------------------------------------------------------------
knitr::include_graphics("https://user-images.githubusercontent.com/1825120/81125221-75c06780-8f2f-11ea-8cea-ad6322ef99e7.png")

## ----eval=FALSE---------------------------------------------------------------
# # source: https://www.robinlovelace.net/presentations/munster.html#31
# points2line_trajectory = function(p) {
#   c = st_coordinates(p)
#   i = seq(nrow(p) - 2)
#   l = purrr::map(i, ~ sf::st_linestring(c[.x:(.x + 1), ]))
#   lfc = sf::st_sfc(l)
#   a = seq(length(lfc)) + 1 # sequence to subset
#   p_data = cbind(sf::st_set_geometry(p[a, ], NULL))
#   sf::st_sf(p_data, geometry = lfc)
# }
# r = points2line_trajectory(trace_projected)
# # summary(st_length(r)) # mean distance is 1m! Doesn't make sense, need to create segments
# s = slope_raster(r, e = e)
# slope_summary = data.frame(min = min(s), mean = mean(s), max = max(s), stdev = sd(s))
# slope_summary = slope_summary * 100
# knitr::kable(slope_summary, digits = 1)

## ----eval=FALSE, echo=FALSE---------------------------------------------------
# # failed tests
# raster::extract(e, trace_projected)
# raster::writeRaster(e, "e.tif")
# e_terra = terra::rast("e.tif")
# terra::crs(e_terra)
# v = terra::vect("vignettes/3DGRT_TRACES_EPSG25830_v2.shp")
# e_wgs = terra::project(e_terra, v)
# e_stars = stars::st_as_stars(e)
# e_wgs = sf::st_transform(e_stars, 4326)
# stars::write_stars(e_wgs, "e_wgs.tif")
# e2 = raster::raster("e_wgs.tif")
# raster::plot(e)
# plot(trace$geometry, add = TRUE)

## ----echo=FALSE, eval=FALSE---------------------------------------------------
# # discarded way:
# remotes::install_github("jhollist/elevatr")
# sp_bbox = sp::bbox(sf::as_Spatial(sf::st_transform(trace, 4326)))
# e = elevatr::get_aws_terrain(locations = sp_bbox, prj = "+init:4326")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width=7, fig.height=5, fig.align = "center",
  eval = FALSE
)

## ----setup1, warning=FALSE, message = FALSE-----------------------------------
# library(dplyr)
# library(sf)
# # remotes::install_github("ITSLeeds/osmextract")
# library(osmextract)

## ----get_iow, warning=FALSE, message = FALSE----------------------------------
# # get the network
# iow_osm = oe_get("Isle of Wight", provider = "geofabrik", stringsAsFactors = FALSE,
#                  quiet = FALSE, force_download = TRUE, force_vectortranslate = TRUE) # 7 MB
# 
# # filter the major roads
# iow_network = iow_osm %>%
#   dplyr::filter(highway %in% c('primary', "primary_link", 'secondary',"secondary_link",
#                                'tertiary', "tertiary_link", "trunk", "trunk_link",
#                                "residential", "cycleway", "living_street", "unclassified",
#                                "motorway", "motorway_link", "pedestrian", "steps", "track")) #remove: "service"

## ----setup2, warning=FALSE, message = FALSE-----------------------------------
# # remotes::install_github("ropensci/stplanr")
# library(stplanr)

## ----filter-------------------------------------------------------------------
# # filter unconnected roads
# iow_network$group = rnet_group(iow_network)
# iow_network_clean = iow_network %>% filter(group == 1) # the network with more connected segments

## ----breaking, warning=FALSE, message = FALSE---------------------------------
# iow_network_segments = rnet_breakup_vertices(iow_network_clean)

## ----import_dem, message=FALSE------------------------------------------------
# # Import and plot DEM
# u = "https://github.com/U-Shift/Declives-RedeViaria/releases/download/0.2/IsleOfWightNASA_clip.tif"
# f = basename(u)
# download.file(url = u, destfile = f, mode = "wb")
# dem = raster::raster(f)
# # res(dem) #27m of resolution
# network = iow_network_segments
# 
# library(raster)
# plot(dem)
# plot(sf::st_geometry(network), add = TRUE) #check if they overlay

## ----slopes_values------------------------------------------------------------
# # Get the slope value for each segment (abs), using slopes package
# library(slopes)
# library(geodist)
# network$slope = slope_raster(network, dem)
# network$slope = network$slope*100 #percentage
# summary(network$slope) #check the values

## ----classify-----------------------------------------------------------------
# # Classify slopes
# network$slope_class = network$slope %>%
#   cut(
#     breaks = c(0, 3, 5, 8, 10, 20, Inf),
#     labels = c("0-3: flat", "3-5: mild", "5-8: medium", "8-10: hard",
#                "10-20: extreme", ">20: impossible"),
#     right = F
#   )
# round(prop.table(table(network$slope_class))*100,1)

## ----map, message = FALSE, eval=FALSE-----------------------------------------
# # more useful information
# network$length = st_length(network)
# 
# # make an interactive map
# library(tmap)
# palredgreen = c("#267300", "#70A800", "#FFAA00", "#E60000", "#A80000", "#730000") #color palette
# # tmap_mode("view")
# tmap_options(basemaps = leaflet::providers$CartoDB.Positron) #basemap
# 
# slopemap =
#   tm_shape(network) +
#   tm_lines(
#     col = "slope_class",
#     palette = palredgreen,
#     lwd = 2, #line width
#     title.col = "Slope [%]",
#     popup.vars = c("Highway" = "highway",
#                    "Length" = "length",
#                   "Slope: " = "slope",
#                   "Class: " = "slope_class"),
#     popup.format = list(digits = 1),
#     # id = "slope"
#     id = "name" #if it gets too memory consuming, delete this line
#   )
# 
# slopemap

## ----export, echo=TRUE, eval=FALSE--------------------------------------------
# #export to html
# tmap_save(slopemap, "html/SlopesIoW.html")
# 
# # export information as geopackage
# st_write(network, "shapefiles/SlopesIoW.gpkg", append=F)

## ----tidyup, include=FALSE----------------------------------------------------
# rm(iow_osm,iow_network_clean,iow_network_segments, iow_network, slopemap)
# file.remove(f) # remove the file, tidy up

#' Read the bundled Lisbon DEM as a SpatRaster
#'
#' Returns a `SpatRaster` (terra package) of the Digital Elevation Model
#' for central Lisbon, Portugal, bundled with the slopes package.
#'
#' @return A `SpatRaster` object with 133 rows, 200 columns, and 1 elevation layer.
#' @export
#' @examples
#' dem_lisbon()
dem_lisbon <- function() {
  tif <- system.file("dem_lisbon.tif", package = "slopes")
  if (!nzchar(tif)) stop("dem_lisbon.tif not found in slopes package installation.")
  terra::rast(tif)
}

#' Cycle route data
#' @format An sf object
"cyclestreets_route"

#' Lisbon road network
#' @format An sf object
"lisbon_road_network"

#' Lisbon road segment
#' @format An sf object
"lisbon_road_segment"

#' Lisbon road segment 3D
#' @format An sf object
"lisbon_road_segment_3d"

#' Lisbon road segment XYZ
#' @format An sf object
"lisbon_road_segment_xyz_mapbox"

#' Lisbon route data
#' @format An sf object
"lisbon_route"

#' Lisbon route 3D
#' @format An sf object
"lisbon_route_3d"

#' Lisbon route XYZ
#' @format An sf object
"lisbon_route_xyz_mapbox"

#' Magnolia coordinates
#' @format A data frame
"magnolia_xy"

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(slopes)

## ----cumulative-slopes, fig.cap="Illustration of the importance of slope length. 4 segments with an 8% gradient is not the same as a single segment with a gradient of 8%.", out.width="40%", echo=FALSE----
knitr::include_graphics("SLOPES-commulative-slope-1.png")

## -----------------------------------------------------------------------------
x = c(0, 2, 3, 4, 5, 9)
y = c(0, 0, 0, 0, 0, 9)
z = c(1, 2, 2, 4, 3, 1) / 10
m = cbind(x, y, z)
d = sequential_dist(m = m, lonlat = FALSE)

slopes::slope_distance_weighted(d = d, elevations = z)
slopes::slope_distance_mean(d = d, elevations = z)

## ----weighted, fig.cap="Illustration of example data that demonstrates distance-weighted mean gradient, used by default in the slopes package."----
plot(x, z, ylim = c(-0.5, 0.5), type = "l")
(gxy = slope_matrix(m, lonlat = FALSE))
abline(h = 0, lty = 2)
points(x[-length(x)], gxy, col = "blue")
title("Distance elevation profile",
  sub = "Points show calculated gradients of subsequent lines")

#' Plot distance-elevation profile with slope coloring
#'
#' Creates a distance-elevation plot with segments colored by slope gradient.
#'
#' @param d Vector of cumulative distances
#' @param z Vector of elevation values
#' @param fill Logical, whether to fill segments with slope colors (default: TRUE)
#' @param horiz Logical, whether legend should be horizontal (default: FALSE)
#' @param pal Color palette for slope visualization (default: NULL, uses slopes_palette)
#' @param ... Additional arguments passed to graphics functions
#' @param legend_position Position of legend (default: "top")
#' @param col Color of the elevation profile line (default: "black")
#' @param cex Character expansion factor for legend text (default: 0.9)
#' @param bg Background color for legend (default: semi-transparent white)
#' @param title Title for the legend (default: "Slope colors (percentage gradient)")
#' @param brks Vector of slope break points for coloring (default: c(3, 6, 10, 20, 40, 100))
#' @param seq_brks Sequence of breaks to show in legend (default: NULL, auto-generated)
#' @param ncol Number of columns in legend (default: 4)
#' @return NULL (creates plot as side effect)
#' @export
plot_dz <- function(
    d, z, fill = TRUE, horiz = FALSE, pal = NULL, ...,
    legend_position = "top", col = "black", cex = 0.9,
    bg = grDevices::rgb(1, 1, 1, 0.8),
    title = "Slope colors (percentage gradient)",
    brks = c(3, 6, 10, 20, 40, 100),
    seq_brks = NULL,
    ncol = 4
) {
  # Make breaks
  b <- make_breaks(brks)
  # Use palette of correct length
  if (is.null(pal)) pal <- slopes_palette(n = length(b) - 1)
  # Ensure pal matches intervals
  if (length(pal) != length(b) - 1) {
    pal <- grDevices::colorRampPalette(pal)(length(b) - 1)
  }
  graphics::plot(d, z, type = "l", col = "brown", lwd = 2)
  if (fill) {
    g <- slope_vector(x = d, elevations = z)
    colz <- make_colz(g, b, pal)
    lapply(seq(d)[-(length(d))], function(i) {
      graphics::polygon(
        x = c(d[i:(i + 1)], d[(i + 1):i]),
        y = c(z[i], z[i + 1], 0, 0),
        col = colz[i],
        border = NA
      )
    })
    graphics::lines(d, z, col = col, lwd = 2)
    if(is.null(seq_brks)) seq_brks <- seq(from = 3, to = length(b) - 2)
    s <- c(seq_brks[-(length(seq_brks) / 2) -1], max(seq_brks) + 1)
    graphics::legend(x = legend_position, legend = b[s] * 100, fill = pal[s],
                     ..., bg = bg, title = title, horiz = horiz,
                     ncol = ncol, cex = cex)
  }
}

#' Plot elevation profile with slope coloring
#'
#' Creates an elevation profile plot from route geometries with XYZ coordinates,
#' with segments colored according to slope gradient.
#'
#' @param route_xyz An sf object containing linestring geometries with XYZ coordinates
#' @param lonlat Logical, whether coordinates are longitude/latitude (default: auto-detected)
#' @param fill Logical, whether to fill segments with slope colors (default: TRUE)
#' @param horiz Logical, whether legend should be horizontal (default: FALSE)
#' @param pal Color palette for slope visualization (default: NULL, uses slopes_palette)
#' @param legend_position Position of legend (default: "top")
#' @param col Color of the elevation profile line (default: "black")
#' @param cex Character expansion factor for legend text (default: 0.9)
#' @param bg Background color for legend (default: semi-transparent white)
#' @param title Title for the legend (default: "Slope colors (percentage gradient)")
#' @param brks Vector of slope break points for coloring (default: c(3, 6, 10, 20, 40, 100))
#' @param seq_brks Sequence of breaks to show in legend (default: auto-generated)
#' @param ncol Number of columns in legend (default: 4)
#' @param ... Additional arguments passed to plot_dz
#' @return NULL (creates plot as side effect)
#' @export
plot_slope <- function(
    route_xyz,
    lonlat = sf::st_is_longlat(route_xyz),
    fill = TRUE,
    horiz = FALSE,
    pal = NULL,
    legend_position = "top",
    col = "black",
    cex = 0.9,
    bg = grDevices::rgb(1, 1, 1, 0.8),
    title = "Slope colors (percentage gradient)",
    brks = c(3, 6, 10, 20, 40, 100),
    seq_brks = seq(from = 3, to = length(brks) * 2 - 2),
    ncol = 4,
    ...
) {
  if (is.null(pal)) pal <- slopes_palette(n = length(brks) - 1)
  if(is.na(lonlat)) {
    stop(
      "CRS of routes not known. Set the CRS, e.g. as follows:\n",
      "sf::st_crs(routes) = 4326 # if the routes are in lon/lat coordinates"
    )
  }
  dz <- distance_z(route_xyz, lonlat = lonlat)
  # Corrected call
  plot_dz(
    d = dz$d,
    z = dz$z,
    fill = fill,
    horiz = horiz,
    pal = pal,
    legend_position = legend_position,
    col = col,
    cex = cex,
    bg = bg,
    title = title,
    brks = brks,
    seq_brks = seq_brks,
    ncol = ncol,
    ...
  )
}

#' Create color palette for slope visualization
#'
#' Creates or processes color palettes for slope gradient visualization.
#'
#' @param pal Color palette (function or character vector)
#' @param b Vector of breaks for color mapping
#' @return Character vector of colors
make_pal <- function(pal, b) {
  if (identical(pal, colorspace::diverging_hcl)) {
    pal <- slopes_palette(n = length(b) - 1)
  } else if (is.function(pal)) {
    pal <- pal(n = length(b) - 1)
  } else if (is.character(pal)) {
    if (length(pal) < length(b) - 1) {
      pal <- grDevices::colorRampPalette(pal)(length(b) - 1)
    } else {
      pal <- pal[seq_len(length(b) - 1)]
    }
  } else {
    stop("pal must be a function or a character vector of hex colors")
  }
  pal
}

#' Extract distance and elevation data from route
#'
#' Extracts cumulative distance and elevation vectors from route XYZ coordinates.
#'
#' @param route_xyz An sf object with XYZ coordinates
#' @param lonlat Logical, whether coordinates are longitude/latitude
#' @return List with components d (distances) and z (elevations)
distance_z <- function(route_xyz, lonlat) {
  m <- sf::st_coordinates(route_xyz)
  d <- cumsum(sequential_dist(m, lonlat = lonlat))
  d <- c(0, d)
  z <- m[, 3]
  list(d = d, z = z)
}

#' Create slope breaks for color mapping
#'
#' Creates symmetric slope breaks around zero for color classification.
#'
#' @param brks Vector of positive slope break values (as percentages)
#' @return Vector of slope breaks including negative values and zero
make_breaks <- function(brks) {
  n <- brks
  n <- c(-rev(n), 0, (n))
  b <- n / 100
  b
}

#' Assign colors to slope values
#'
#' Maps slope gradient values to colors based on break points.
#'
#' @param g Vector of slope gradient values
#' @param b Vector of break points
#' @param pal Vector of colors corresponding to breaks
#' @return Character vector of colors for each slope value
make_colz <- function(g, b, pal) {
  colz <- cut(
    x = g,
    breaks = b,
    labels = pal
  )
  as.character(colz)
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----eval=FALSE---------------------------------------------------------------
# # install.packages("remotes")
# remotes::install_github("ropensci/slopes")

## ----eval=FALSE---------------------------------------------------------------
# # install.packages("remotes")
# remotes::install_github("ropensci/slopes", dependencies = "Suggests")

## ----eval=FALSE---------------------------------------------------------------
# usethis::edit_r_environ()
# # Then add the following line to the file that opens:
# # MAPBOX_API_KEY=xxxxx # replace XXX with your api key

## -----------------------------------------------------------------------------
library(slopes)
library(sf)

# Load example data
data(lisbon_route)
dem_lisbon <- dem_lisbon()

## -----------------------------------------------------------------------------
sf_linestring_xyz_local <- elevation_add(lisbon_route, dem = dem_lisbon)
head(sf::st_coordinates(sf_linestring_xyz_local))

## ----eval=FALSE---------------------------------------------------------------
# # Requires a MapBox API key and the ceramic package
# # sf_linestring_xyz_mapbox = elevation_add(lisbon_route)
# # head(sf::st_coordinates(sf_linestring_xyz_mapbox))

## -----------------------------------------------------------------------------
slope <- slope_xyz(sf_linestring_xyz_local)
slope

## -----------------------------------------------------------------------------
# ensure a palette and breaks exist
brks <- c(3, 6, 10, 20, 40, 100)
pal <- slopes_palette(length(brks) - 1)

plot_slope(sf_linestring_xyz_local, pal = pal, brks = brks)

## -----------------------------------------------------------------------------
lisbon_route_xyz <- elevation_add(lisbon_route, dem = dem_lisbon())
lisbon_route_segments_xyz <- route_to_segments(lisbon_route_xyz)
lisbon_route_segments_xyz$slope <- slope_xyz(lisbon_route_segments_xyz)
summary(lisbon_route_segments_xyz$slope)

## -----------------------------------------------------------------------------
plot(st_geometry(lisbon_route_segments_xyz),
  col = heat.colors(length(lisbon_route_segments_xyz$slope))[rank(lisbon_route_segments_xyz$slope)],
  lwd = 3, main = "Slope by vertex segments"
)

## ----warning=FALSE------------------------------------------------------------
lisbon_route_100m <- stplanr::line_segment(lisbon_route, segment_length = 100)
lisbon_route_100m_xyz <- elevation_add(lisbon_route_100m, dem = dem_lisbon())
lisbon_route_100m_xyz$slope <- slope_xyz(lisbon_route_100m_xyz)
summary(lisbon_route_100m_xyz$slope)

## -----------------------------------------------------------------------------
plot(st_geometry(lisbon_route_100m_xyz),
  col = heat.colors(length(lisbon_route_100m_xyz$slope))[rank(lisbon_route_100m_xyz$slope)],
  lwd = 3, main = "Slope by 100 m segments"
)

---
title: "Example: gradients of a road network for a given city"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Example: gradients of a road network for a given city}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width=7, fig.height=5, fig.align = "center",
  eval = FALSE
)
```

An example of the demand for data provided by the `slopes` package is a map showing gradients across São Paulo (Brazil, see image below), with a simplistic classification for cycling difficulty.

![using slopes() to create a road network gradient for cycling for São Paulo (Brazil)](https://pbs.twimg.com/media/ErJ2dr8WMAIHwMn?format=jpg&name=small)

This vignette will guide through the production of an interactive slope map for a road network, using `slopes`, `osmextract`, `sf`, `stplanr` and `tmap`.  

For the convenience of sample, we will use [Isle of Wight](https://en.wikipedia.org/wiki/Isle_of_Wight) case, with 384 km^2^. See [Other examples] below.

This will follow three steps:

1.  Download of road network from [OpenStreetMap](https://www.openstreetmap.org/)
2.  Prepare the network
3.  Compute slopes and export the map in html


## Extract the OSM network from geofabrik

For this step you may use `osmextract` [package](https://ropensci.github.io/osmextract/articles/osmextract.html) which downloads the most recent information available at OSM (https://download.geofabrik.de/index.html) and converts to _GeoPackage_ (.gpkg), the equivalent to _shapefile_.

```{r setup1, warning=FALSE, message = FALSE}
library(dplyr)
library(sf)
# remotes::install_github("ITSLeeds/osmextract")
library(osmextract)
```

```{r get_iow, warning=FALSE, message = FALSE}
# get the network
iow_osm = oe_get("Isle of Wight", provider = "geofabrik", stringsAsFactors = FALSE, 
                 quiet = FALSE, force_download = TRUE, force_vectortranslate = TRUE) # 7 MB

# filter the major roads
iow_network = iow_osm %>% 
  dplyr::filter(highway %in% c('primary', "primary_link", 'secondary',"secondary_link", 
                               'tertiary', "tertiary_link", "trunk", "trunk_link", 
                               "residential", "cycleway", "living_street", "unclassified", 
                               "motorway", "motorway_link", "pedestrian", "steps", "track")) #remove: "service"
```

## Clean the road network

These are optional steps that give better results, although they may slow down the process since they increase the number of segments present in the network.

### Filter the unconnected segments

The [`rnet_group()`](https://docs.ropensci.org/stplanr/reference/rnet_group.html) function from `stplanar` package assesses the connectivity of each segment assigns a group number (similar to a clustering process). Then we may filter the main group, the one with more connected segments.

```{r setup2, warning=FALSE, message = FALSE}
# remotes::install_github("ropensci/stplanr")
library(stplanr)
```

```{r filter}
# filter unconnected roads
iow_network$group = rnet_group(iow_network)
iow_network_clean = iow_network %>% filter(group == 1) # the network with more connected segments
```

### Break the segments on vertices

A very long segment will have an assigned average slope, but a very long segment can be broken into its nodes and have its own slope in each part of the segment. On one hand, we want the segments to break at their nodes. On the other hand, we don't want artificial *nodes* to be created where two lines cross, in particular where they have different **z** levels (_eg._ *brunels*: bridges and tunnels).  

The [`rnet_breakup_vertices`](https://docs.ropensci.org/stplanr/reference/rnet_breakup_vertices.html) from `stplanr` breaks the segments at their inner vertices, preserving the **brunels**.

```{r breaking, warning=FALSE, message = FALSE}
iow_network_segments = rnet_breakup_vertices(iow_network_clean)
```

In this case, there are around 1.6 x the number of segments than in the original network. 

<!-- `r # round(nrow(iow_network_segments)/nrow(iow_network_clean),2)`x segments than the original network. -->

## Get slope values for each segment

For this case we will use `slope_raster()` [function](https://ropensci.github.io/slopes/reference/slope_raster.html), to retrieve the z values from a digital elevation model. This raster was obtained from STRM NASA mission.

The **SRTM** (*Shuttle Radar Topography Mission*) NASA’s mission provides [freely available](https://gisgeography.com/srtm-shuttle-radar-topography-mission/) worldwide DEM, with a resolution of 25 to 30m and with a vertical accuracy of 16m - [more](https://portal.opentopography.org/raster?opentopoID=OTSRTM.082015.4326.1). The resolution for USA might be better.

Alternatively, **COPERNICUS** ESA's mission also provides [freely available](https://land.copernicus.eu/en/products/products-that-are-no-longer-disseminated-on-the-clms-website) DEM for all Europe, with a 25m resolution and a vertical accuracy of 7m - [more](https://land.copernicus.eu/en/news).

Depending of how large is your road network, you can use `elevation_add()` [function](https://ropensci.github.io/slopes/reference/elevation_add.html) - this will require a valid [Mapbox api key](https://docs.mapbox.com/api/overview/).

```{r import_dem, message=FALSE}
# Import and plot DEM
u = "https://github.com/U-Shift/Declives-RedeViaria/releases/download/0.2/IsleOfWightNASA_clip.tif"
f = basename(u)
download.file(url = u, destfile = f, mode = "wb")
dem = raster::raster(f)
# res(dem) #27m of resolution
network = iow_network_segments

library(raster)
plot(dem)
plot(sf::st_geometry(network), add = TRUE) #check if they overlay
```

All the required data is prepared to estimate the road segments' gradient.

```{r slopes_values}
# Get the slope value for each segment (abs), using slopes package
library(slopes)
library(geodist)
network$slope = slope_raster(network, dem)
network$slope = network$slope*100 #percentage
summary(network$slope) #check the values
```

Half of the road segments in Isle of Wight have a gradient below 3.1%.
<!-- `r # round(median(network$slope),1)`%. -->

We will adopt a simplistic qualitative classification for **cycling effort uphill**, and compare the number of segments in each class.

```{r classify}
# Classify slopes
network$slope_class = network$slope %>%
  cut(
    breaks = c(0, 3, 5, 8, 10, 20, Inf),
    labels = c("0-3: flat", "3-5: mild", "5-8: medium", "8-10: hard", 
               "10-20: extreme", ">20: impossible"),
    right = F
  )
round(prop.table(table(network$slope_class))*100,1)
```
<!-- It means that **`r # round(prop.table(table(network$slope_class))[[1]]*100)`%** of the roads are flat or almost flat (0-3%) and about **`r # round(prop.table(table(network$slope_class))[[1]]*100)+round(prop.table(table(network$slope_class))[[2]]*100)`%** of the roads are easily cyclable (0-5%). -->
It means that **49%** of the roads are flat or almost flat (0-3%) and about **75%** of the roads are easily cyclable (0-5%).

Now let us put this information on a map (see [here](https://rpubs.com/RobinLovelace/781081) for interactive version).

```{r map, message = FALSE, eval=FALSE}
# more useful information
network$length = st_length(network)

# make an interactive map
library(tmap)
palredgreen = c("#267300", "#70A800", "#FFAA00", "#E60000", "#A80000", "#730000") #color palette
# tmap_mode("view")
tmap_options(basemaps = leaflet::providers$CartoDB.Positron) #basemap

slopemap =
  tm_shape(network) +
  tm_lines(
    col = "slope_class",
    palette = palredgreen,
    lwd = 2, #line width
    title.col = "Slope [%]",
    popup.vars = c("Highway" = "highway",
                   "Length" = "length",
                  "Slope: " = "slope",
                  "Class: " = "slope_class"),
    popup.format = list(digits = 1),
    # id = "slope"
    id = "name" #if it gets too memory consuming, delete this line
  )

slopemap
```


```{r export, echo=TRUE, eval=FALSE}
#export to html
tmap_save(slopemap, "html/SlopesIoW.html") 

# export information as geopackage
st_write(network, "shapefiles/SlopesIoW.gpkg", append=F)
```

#### Result:

![](https://user-images.githubusercontent.com/1825120/121820334-2435f080-cc8a-11eb-962c-79dcba97e459.png)

-   [Isle of Wight (UK)](http://web.tecnico.ulisboa.pt/~rosamfelix/gis/declives/SlopesIoW.html)

## Other examples

-   [São Paulo (Brazil)](https://web.tecnico.ulisboa.pt/~rosamfelix/gis/declives/DeclivesSaoPaulo.html)
-   [Lisbon (Portugal)](https://web.tecnico.ulisboa.pt/~rosamfelix/gis/declives/DeclivesLisboa.html)
-   [Oporto (Portugal)](https://web.tecnico.ulisboa.pt/~rosamfelix/gis/declives/DeclivesPorto_EU.html)
-   [Leeds (UK)](https://web.tecnico.ulisboa.pt/~rosamfelix/gis/declives/SlopesLeeds.html)
-   [Zurich (CH)](https://web.tecnico.ulisboa.pt/~rosamfelix/gis/declives/SlopesZurich.html)
-   [Paris (FR)](https://web.tecnico.ulisboa.pt/~rosamfelix/gis/declives/SlopesParis_NASA.html)


```{r tidyup, include=FALSE}
rm(iow_osm,iow_network_clean,iow_network_segments, iow_network, slopemap)
file.remove(f) # remove the file, tidy up
```

---
title: "An introduction to slopes"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{An introduction to slopes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: slope-references.bib
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(slopes)
```

## Introduction

Although there are several ways to name "slope", such as "steepness", "hilliness", "inclination", "aspect", "gradient", "declivity", the referred `slopes` in this package can be defined as the "longitudinal gradient" of linear geographic entities, as defined in the context of rivers by[@cohen_global_2018].  

The package was initially developed to research road slopes to support evidence-based sustainable transport policies.
Accounting for gradient when planning for new cycling infrastructure and road space reallocation for walking and cycling can improve outcomes, for example by helping to identify routes that avoid steep hills.
The package can be used to calculate and visualise slopes of rivers and trajectories representing movement on roads of the type published as open data by @ariza-lopez_dataset_2019.

Data on slopes are useful in many fields of research, including [hydrology](https://en.wikipedia.org/wiki/Stream_gradient), natural hazards (including [flooding](https://response.reliefweb.int/afghanistan) and [landslide risk management](https://assets.publishing.service.gov.uk/media/57a08d0740f0b652dd0016f4/R7815-ADD017_col.pdf)), recreational and competitive sports such as [cycling](https://theclimbingcyclist.com/gradients-and-cycling-an-introduction/), [hiking](https://trailism.com/trail-grades/), and [skiing](https://www.snowplaza.co.uk/blog/16682-skiing-steeps-what-does-gradient-mean-ski-piste/).
Slopes are also also important in some branches of [transport and emissions modelling](https://doi.org/10.1016/j.trpro.2016.05.258) and [ecology](https://doi.org/10.1016/j.ncon.2016.10.001).
A growing number of people working with geospatial data require accurate estimates of gradient, including:

- Transport planning practitioners who require accurate estimates of roadway gradient for estimating energy consumption, safety and mode shift potential in hilly cities (such as Lisbon, the case study city used in the examples in the documentation). 
- Vehicle routing software developers, who need to build systems are sensitive to going up or down steep hills (e.g. bicycles, trains, and large trucks), such as active travel planning, logistics, and emergency services.
- Natural hazard researchers and risk assessors require estimates of linear gradient to inform safety and mitigation plans associated with project on hilly terrain.
- Aquatic ecologists, flooding researchers and others, who could benefit from estimates of river gradient to support modelling of storm hydrographs

There likely other domains where slopes could be useful, such as agriculture, geology, and civil engineering.

An example of the demand for data provided by the package is a map showing gradients across Sao Paulo (Brazil, see image below) that has received more than 200 'likes' on Twitter and generated conversations:  `https://twitter.com/DanielGuth/status/1347270685161304069`

![](https://camo.githubusercontent.com/30a3b814dd72aef5b51db635f2ab6e1b6b6c57b856d239822788967a4932d655/68747470733a2f2f7062732e7477696d672e636f6d2f6d656469612f45724a32647238574d414948774d6e3f666f726d61743d6a7067266e616d653d6c61726765){ width=50% } 


## Calculating slopes

The most common slope calculation method is defined by the vertical difference of the final and start point or line height (z1 and z0) divided by the horizontal length that separates them.

$$
s = \Delta z/l
$$

Depending on the purpose of application, it might me relevant to understand how hilliness is estimated.  

![Traffic sign](https://sinalnorte.com/wp-content/uploads/2018/02/A3b.jpg){ width=10% }  

### Measures of route hilliness

There are many ways to measure hilliness, mean distance weighted hilliness being perhaps the most common.
These measures, and their implementation (or current lack thereof) in the package is summarised below.

- **Mean distance weighted gradient**. Perhaps the simplest and most widely applicable measure is the mean gradient of the route. This should be weighted by the distance of each segment. Implemented by default in the `slope_raster()` function.

- **Max gradient**. For activities like cycling, where steep hills have a disproportionate impact, it may be useful to consider the maximum gradient. Not yet implemented.

<!-- Todo: update when we have `method = max` -->

- **Xth percentile gradient**. Since the maximum gradient gives no information about the rest of the route segments, other measures such as the 75th percentile gradient could be more informative. Not yet implemented.

- **Inverted harmonic mean**. If we use the following formula we will get an index that (like the arithmetic mean) makes use of the full dataset, but that is weighted towards the higher gradient segments. Whether this index, the formula of which is shown below, is helpful, remains to be tested. Not yet implemented.

$$
H(x) = 1 - distance.weighted.harmonic.mean(1-x)
$$


### Segments in a route: Cumulative slope 

The length of a segment in a route is also a relevant factor to have in consideration. If it is ok to bike through a segment of 8% with xx length, it is not so ok to bike in four segments in a row like that one (8% for 4xx length), as illustrated bellow.

```{r cumulative-slopes, fig.cap="Illustration of the importance of slope length. 4 segments with an 8% gradient is not the same as a single segment with a gradient of 8%.", out.width="40%", echo=FALSE}
knitr::include_graphics("SLOPES-commulative-slope-1.png")
```


This is accounted for in slope calculation methods that take the distance-weighted mean of slopes.

```{r}
x = c(0, 2, 3, 4, 5, 9)
y = c(0, 0, 0, 0, 0, 9)
z = c(1, 2, 2, 4, 3, 1) / 10
m = cbind(x, y, z)
d = sequential_dist(m = m, lonlat = FALSE)

slopes::slope_distance_weighted(d = d, elevations = z)
slopes::slope_distance_mean(d = d, elevations = z)
```

The slope estimate that results from the distance-weighted mean is lower than the simple mean.
This is common: steep slopes tend to be short!

A graphical representation of the scenario demonstrated above is shown in Figure \@ref(fig:weighted), that shows the relatively long and flat final segment reduces the slope by half.

```{r weighted, fig.cap="Illustration of example data that demonstrates distance-weighted mean gradient, used by default in the slopes package."}
plot(x, z, ylim = c(-0.5, 0.5), type = "l")
(gxy = slope_matrix(m, lonlat = FALSE))
abline(h = 0, lty = 2)
points(x[-length(x)], gxy, col = "blue")
title("Distance elevation profile",
  sub = "Points show calculated gradients of subsequent lines")
```




# References

---
title: "Benchmarking slopes calculation"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Benchmarking slopes calculation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(slopes)
library(bench)
```

# Performance

A benchmark can reveal how many route gradients can be calculated per second
using different interpolation methods:

```{r, results='hide'}
e = dem_lisbon()
r = lisbon_road_network
res = bench::mark(check = FALSE,
  bilinear = slope_raster(r, e),
  simple   = slope_raster(r, e, method = "simple")
)
```

```{r}
res
```


That is approximately

```{r}
round(res$`itr/sec` * nrow(r))
```

routes per second using `bilinear` and `simple` interpolation methods, respectively.

To go faster, you can chose the `simple` method to gain some speed at the expense of accuracy:

```{r, results='hide'}
res2 = bench::mark(check = FALSE,
  bilinear = slope_raster(r, e, method = "bilinear"),
  simple   = slope_raster(r, e, method = "simple")
)
```

```{r}
res2
```

```{r}
round(res2$`itr/sec` * nrow(r))
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slopes.R
\name{slope_vector}
\alias{slope_vector}
\title{Calculate slopes from vector data}
\usage{
slope_vector(x, elevations)
}
\arguments{
\item{x}{Vector of distance values}

\item{elevations}{Vector of elevation values}
}
\value{
Numeric vector of slope values
}
\description{
Calculates slope gradients using vector distance and elevation data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slopes.R
\name{elevation_add}
\alias{elevation_add}
\title{Add elevation data to route linestrings}
\usage{
elevation_add(routes, dem = NULL, method = "bilinear", terra = NULL)
}
\arguments{
\item{routes}{An sf object containing linestring geometries}

\item{dem}{A SpatRaster object containing elevation data (default: NULL for automatic download)}

\item{method}{Method for raster extraction (default: "bilinear")}

\item{terra}{Deprecated. Ignored; terra is always used.}
}
\value{
An sf object with XYZ linestring geometries
}
\description{
Adds elevation (Z) coordinates to linestring geometries using DEM data.
}
\examples{
library(sf)
routes = lisbon_road_network[204, ]
dem = dem_lisbon()
(r3d = elevation_add(routes, dem))
st_z_range(routes)
st_z_range(r3d)
plot(st_coordinates(r3d)[, 3])
plot_slope(r3d)
\dontrun{
# Get elevation data (requires internet connection, ceramic pkg, and API key):
if (requireNamespace("ceramic", quietly = TRUE)) {
  r3d_get = elevation_add(cyclestreets_route)
  plot_slope(r3d_get)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slopes.R
\name{slope_raster}
\alias{slope_raster}
\title{Calculate slopes using raster elevation data}
\usage{
slope_raster(
  routes,
  dem,
  lonlat = sf::st_is_longlat(routes),
  method = "bilinear",
  fun = slope_matrix_weighted,
  terra = NULL,
  directed = FALSE
)
}
\arguments{
\item{routes}{An sf object containing linestring geometries}

\item{dem}{A SpatRaster object (terra package) containing elevation data}

\item{lonlat}{Logical, whether coordinates are longitude/latitude (default: auto-detected)}

\item{method}{Method for raster extraction (default: "bilinear")}

\item{fun}{Function for slope calculation (default: slope_matrix_weighted)}

\item{terra}{Deprecated. Ignored; terra is always used.}

\item{directed}{Logical, whether to calculate directed slopes (default: FALSE)}
}
\value{
Numeric vector of slope values
}
\description{
Calculates slope gradients for routes using digital elevation model (DEM) raster data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/palette.R
\name{slopes_palette}
\alias{slopes_palette}
\title{Get color palette for slopes visualization}
\usage{
slopes_palette(n = 6, palette = "Green-Brown")
}
\arguments{
\item{n}{Number of colors to return (default: 6)}

\item{palette}{Name of the color palette to use (default: "Green-Brown")}
}
\value{
A character vector of color codes
}
\description{
Returns a color palette suitable for visualizing slope data, with options
for different color schemes.
}
\examples{
# Get default Green-Brown palette with 6 colors
slopes_palette()

# Get 4 colors from Green-Brown palette
slopes_palette(n = 4)

# Use a different palette
slopes_palette(n = 5, palette = "Blue-Red")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_slope.R
\name{make_breaks}
\alias{make_breaks}
\title{Create slope breaks for color mapping}
\usage{
make_breaks(brks)
}
\arguments{
\item{brks}{Vector of positive slope break values (as percentages)}
}
\value{
Vector of slope breaks including negative values and zero
}
\description{
Creates symmetric slope breaks around zero for color classification.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slopes.R
\name{route_to_segments}
\alias{route_to_segments}
\title{Split a route into vertex-to-vertex segments}
\usage{
route_to_segments(route_xyz)
}
\arguments{
\item{route_xyz}{An sf object with a single LINESTRING geometry with Z coordinates,
as returned by \code{\link[=elevation_add]{elevation_add()}}.}
}
\value{
An sf object with one LINESTRING feature per vertex-to-vertex segment.
}
\description{
Splits a linestring with XYZ coordinates into individual 2-point segments,
one per consecutive vertex pair. Useful for computing per-segment slopes
with \code{\link[=slope_xyz]{slope_xyz()}}.
}
\examples{
route_xyz = elevation_add(lisbon_route, dem = dem_lisbon())
segs = route_to_segments(route_xyz)
segs$slope = slope_xyz(segs)
summary(segs$slope)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cyclestreets_route}
\alias{cyclestreets_route}
\title{Cycle route data}
\format{
An sf object
}
\usage{
cyclestreets_route
}
\description{
Cycle route data
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{lisbon_road_segment}
\alias{lisbon_road_segment}
\title{Lisbon road segment}
\format{
An sf object
}
\usage{
lisbon_road_segment
}
\description{
Lisbon road segment
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slopes.R
\name{slope_distance_weighted}
\alias{slope_distance_weighted}
\title{Calculate distance-weighted slopes}
\usage{
slope_distance_weighted(d, elevations, directed = FALSE)
}
\arguments{
\item{d}{Vector of distance values between points}

\item{elevations}{Vector of elevation values}

\item{directed}{Logical, whether to calculate directed slopes (default: FALSE)}
}
\value{
Numeric value representing the weighted slope
}
\description{
Applies distance-based weighting to slope calculations for more accurate results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_slope.R
\name{make_pal}
\alias{make_pal}
\title{Create color palette for slope visualization}
\usage{
make_pal(pal, b)
}
\arguments{
\item{pal}{Color palette (function or character vector)}

\item{b}{Vector of breaks for color mapping}
}
\value{
Character vector of colors
}
\description{
Creates or processes color palettes for slope gradient visualization.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_slope.R
\name{plot_slope}
\alias{plot_slope}
\title{Plot elevation profile with slope coloring}
\usage{
plot_slope(
  route_xyz,
  lonlat = sf::st_is_longlat(route_xyz),
  fill = TRUE,
  horiz = FALSE,
  pal = NULL,
  legend_position = "top",
  col = "black",
  cex = 0.9,
  bg = grDevices::rgb(1, 1, 1, 0.8),
  title = "Slope colors (percentage gradient)",
  brks = c(3, 6, 10, 20, 40, 100),
  seq_brks = seq(from = 3, to = length(brks) * 2 - 2),
  ncol = 4,
  ...
)
}
\arguments{
\item{route_xyz}{An sf object containing linestring geometries with XYZ coordinates}

\item{lonlat}{Logical, whether coordinates are longitude/latitude (default: auto-detected)}

\item{fill}{Logical, whether to fill segments with slope colors (default: TRUE)}

\item{horiz}{Logical, whether legend should be horizontal (default: FALSE)}

\item{pal}{Color palette for slope visualization (default: NULL, uses slopes_palette)}

\item{legend_position}{Position of legend (default: "top")}

\item{col}{Color of the elevation profile line (default: "black")}

\item{cex}{Character expansion factor for legend text (default: 0.9)}

\item{bg}{Background color for legend (default: semi-transparent white)}

\item{title}{Title for the legend (default: "Slope colors (percentage gradient)")}

\item{brks}{Vector of slope break points for coloring (default: c(3, 6, 10, 20, 40, 100))}

\item{seq_brks}{Sequence of breaks to show in legend (default: auto-generated)}

\item{ncol}{Number of columns in legend (default: 4)}

\item{...}{Additional arguments passed to plot_dz}
}
\value{
NULL (creates plot as side effect)
}
\description{
Creates an elevation profile plot from route geometries with XYZ coordinates,
with segments colored according to slope gradient.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slopes.R
\name{slope_xyz}
\alias{slope_xyz}
\title{Calculate slopes from XYZ coordinate data}
\usage{
slope_xyz(
  route_xyz,
  fun = slope_matrix_weighted,
  lonlat = TRUE,
  directed = FALSE
)
}
\arguments{
\item{route_xyz}{An sf object or data frame with XYZ coordinates}

\item{fun}{Function for slope calculation (default: slope_matrix_weighted)}

\item{lonlat}{Logical, whether coordinates are longitude/latitude (default: TRUE)}

\item{directed}{Logical, whether to calculate directed slopes (default: FALSE)}
}
\value{
Numeric vector of slope values
}
\description{
Calculates slope gradients from linestring geometries with XYZ coordinates.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_slope.R
\name{plot_dz}
\alias{plot_dz}
\title{Plot distance-elevation profile with slope coloring}
\usage{
plot_dz(
  d,
  z,
  fill = TRUE,
  horiz = FALSE,
  pal = NULL,
  ...,
  legend_position = "top",
  col = "black",
  cex = 0.9,
  bg = grDevices::rgb(1, 1, 1, 0.8),
  title = "Slope colors (percentage gradient)",
  brks = c(3, 6, 10, 20, 40, 100),
  seq_brks = NULL,
  ncol = 4
)
}
\arguments{
\item{d}{Vector of cumulative distances}

\item{z}{Vector of elevation values}

\item{fill}{Logical, whether to fill segments with slope colors (default: TRUE)}

\item{horiz}{Logical, whether legend should be horizontal (default: FALSE)}

\item{pal}{Color palette for slope visualization (default: NULL, uses slopes_palette)}

\item{...}{Additional arguments passed to graphics functions}

\item{legend_position}{Position of legend (default: "top")}

\item{col}{Color of the elevation profile line (default: "black")}

\item{cex}{Character expansion factor for legend text (default: 0.9)}

\item{bg}{Background color for legend (default: semi-transparent white)}

\item{title}{Title for the legend (default: "Slope colors (percentage gradient)")}

\item{brks}{Vector of slope break points for coloring (default: c(3, 6, 10, 20, 40, 100))}

\item{seq_brks}{Sequence of breaks to show in legend (default: NULL, auto-generated)}

\item{ncol}{Number of columns in legend (default: 4)}
}
\value{
NULL (creates plot as side effect)
}
\description{
Creates a distance-elevation plot with segments colored by slope gradient.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slope_get.R
\name{slope_matrix_to_raster}
\alias{slope_matrix_to_raster}
\title{Convert slope matrix to SpatRaster}
\usage{
slope_matrix_to_raster(x)
}
\arguments{
\item{x}{A matrix, SpatRaster, or legacy RasterLayer object}
}
\value{
A SpatRaster object
}
\description{
Converts a slope matrix or a legacy RasterLayer to a SpatRaster (terra).
Accepts \code{SpatRaster}, legacy \verb{Raster*}, or a plain matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slopes.R
\name{elevation_extract}
\alias{elevation_extract}
\title{Extract elevation values from coordinates}
\usage{
elevation_extract(m, dem, method = "bilinear", terra = NULL)
}
\arguments{
\item{m}{Matrix or sf object with coordinates}

\item{dem}{A SpatRaster (or legacy RasterLayer) containing elevation data}

\item{method}{Method for raster extraction (default: "bilinear")}

\item{terra}{Deprecated. Ignored; terra is always used.}
}
\value{
Numeric vector of elevation values
}
\description{
Extracts elevation values from a DEM raster at specified coordinate locations.
Accepts both \code{SpatRaster} (terra) and legacy \verb{Raster*} (raster) objects;
legacy objects are automatically converted to \code{SpatRaster}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_slope.R
\name{distance_z}
\alias{distance_z}
\title{Extract distance and elevation data from route}
\usage{
distance_z(route_xyz, lonlat)
}
\arguments{
\item{route_xyz}{An sf object with XYZ coordinates}

\item{lonlat}{Logical, whether coordinates are longitude/latitude}
}
\value{
List with components d (distances) and z (elevations)
}
\description{
Extracts cumulative distance and elevation vectors from route XYZ coordinates.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slopes.R
\name{slope_matrices}
\alias{slope_matrices}
\title{Calculate slopes for multiple coordinate matrices}
\usage{
slope_matrices(m_xyz_split, fun = slope_matrix_weighted, ...)
}
\arguments{
\item{m_xyz_split}{List of coordinate matrices with elevation data}

\item{fun}{Function to apply for slope calculation (default: slope_matrix_weighted)}

\item{...}{Additional arguments passed to the slope function}
}
\value{
Numeric vector of slope values for all matrices
}
\description{
Applies slope calculation function to a list of coordinate matrices.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slope_get.R
\name{slope_xyz_simple}
\alias{slope_xyz_simple}
\title{Extract XYZ coordinates from SpatRaster or matrix}
\usage{
slope_xyz_simple(x)
}
\arguments{
\item{x}{A SpatRaster, legacy RasterLayer, or matrix object}
}
\value{
A data frame with x, y, z coordinates
}
\description{
Simplifies raster or matrix data to XYZ coordinate format.
Accepts \code{SpatRaster} (terra), legacy \verb{Raster*} objects, or a plain matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slope_get.R
\name{elevation_get}
\alias{elevation_get}
\title{Get elevation data for routes}
\usage{
elevation_get(routes, ...)
}
\arguments{
\item{routes}{An sf object containing linestring geometries}

\item{...}{Additional arguments passed to ceramic::cc_elevation}
}
\value{
A SpatRaster covering the routes
}
\description{
Downloads elevation data using the ceramic package for given routes.
Returns a \code{SpatRaster} object (terra package).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slopes.R
\name{slope_matrix_weighted}
\alias{slope_matrix_weighted}
\title{Calculate weighted slopes from coordinate matrix}
\usage{
slope_matrix_weighted(m, elevations = m[, 3], lonlat = TRUE, directed = FALSE)
}
\arguments{
\item{m}{Matrix of coordinates (x, y, z)}

\item{elevations}{Vector of elevation values (default: third column of m)}

\item{lonlat}{Logical, whether coordinates are longitude/latitude (default: TRUE)}

\item{directed}{Logical, whether to calculate directed slopes (default: FALSE)}
}
\value{
Numeric value representing the weighted slope
}
\description{
Applies distance-based weighting to slope calculations from coordinate matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slopes.R
\name{slope_distance}
\alias{slope_distance}
\title{Calculate slopes using distance data}
\usage{
slope_distance(d, elevations)
}
\arguments{
\item{d}{Vector of distance values between points}

\item{elevations}{Vector of elevation values}
}
\value{
Numeric vector of slope values
}
\description{
Calculates slope gradients from distance and elevation vectors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slope_get.R
\name{sf_mid_ext_lonlat}
\alias{sf_mid_ext_lonlat}
\title{Extract midpoint and extent from routes in lonlat}
\usage{
sf_mid_ext_lonlat(routes)
}
\arguments{
\item{routes}{An sf object containing linestring geometries}
}
\value{
A list with midpoint coordinates and width/height dimensions
}
\description{
Internal helper function to get midpoint and extent of routes in lon/lat coordinates.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{lisbon_route_xyz_mapbox}
\alias{lisbon_route_xyz_mapbox}
\title{Lisbon route XYZ}
\format{
An sf object
}
\usage{
lisbon_route_xyz_mapbox
}
\description{
Lisbon route XYZ
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slopes.R
\name{slope_distance_mean}
\alias{slope_distance_mean}
\title{Calculate mean slope using distance weighting}
\usage{
slope_distance_mean(d, elevations, directed = FALSE)
}
\arguments{
\item{d}{Vector of distance values between points}

\item{elevations}{Vector of elevation values}

\item{directed}{Logical, whether to calculate directed slopes (default: FALSE)}
}
\value{
Numeric value representing the mean slope
}
\description{
Computes the mean slope across segments using distance-weighted averaging.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slopes.R
\name{slope_matrix}
\alias{slope_matrix}
\title{Calculate slopes from coordinate matrix}
\usage{
slope_matrix(m, elevations = m[, 3], lonlat = TRUE)
}
\arguments{
\item{m}{Matrix of coordinates (x, y, z)}

\item{elevations}{Vector of elevation values (default: third column of m)}

\item{lonlat}{Logical, whether coordinates are longitude/latitude (default: TRUE)}
}
\value{
Numeric vector of slope values
}
\description{
Calculates slope gradients from a matrix of coordinates and elevation data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{magnolia_xy}
\alias{magnolia_xy}
\title{Magnolia coordinates}
\format{
A data frame
}
\usage{
magnolia_xy
}
\description{
Magnolia coordinates
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{dem_lisbon}
\alias{dem_lisbon}
\title{Read the bundled Lisbon DEM as a SpatRaster}
\usage{
dem_lisbon()
}
\value{
A \code{SpatRaster} object with 133 rows, 200 columns, and 1 elevation layer.
}
\description{
Returns a \code{SpatRaster} (terra package) of the Digital Elevation Model
for central Lisbon, Portugal, bundled with the slopes package.
}
\examples{
dem_lisbon()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_slope.R
\name{make_colz}
\alias{make_colz}
\title{Assign colors to slope values}
\usage{
make_colz(g, b, pal)
}
\arguments{
\item{g}{Vector of slope gradient values}

\item{b}{Vector of break points}

\item{pal}{Vector of colors corresponding to breaks}
}
\value{
Character vector of colors for each slope value
}
\description{
Maps slope gradient values to colors based on break points.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slopes.R
\name{sequential_dist}
\alias{sequential_dist}
\title{Calculate sequential distances between points}
\usage{
sequential_dist(m, lonlat = TRUE)
}
\arguments{
\item{m}{Matrix of coordinates (x, y)}

\item{lonlat}{Logical, whether coordinates are longitude/latitude (default: TRUE)}
}
\value{
Numeric vector of distances between consecutive points
}
\description{
Calculates distances between consecutive points in a coordinate matrix.
}
